//
// Copyright (c) 2016, MindFusion LLC - Bulgaria.
//

using System;
using System.Collections;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Text;
using System.Windows.Forms;

using MindFusion.HolidayProviders;


namespace MindFusion.Scheduling.WinForms.Samples.CS.Holidays
{
	/// <summary>
	/// Summary description for MainForm.
	/// </summary>
	public partial class MainForm : Form
	{
		/// <summary>
		/// Initializes a new instance of the MainForm class.
		/// </summary>
		public MainForm()
		{
			InitializeComponent();

			_appPath = System.IO.Path.GetDirectoryName(Application.ExecutablePath);

			_lbInfo.DocumentText = Properties.Resources.Info;
			_lbInfo.DocumentCompleted += new WebBrowserDocumentCompletedEventHandler(OnDocumentCompleted);

			calendar.Date = DateTime.Today;
			calendar.MonthSettings.DaySettings.HeaderSize = 0;
			UpdateHolidays();
		}

		private void OnDocumentCompleted(object sender, WebBrowserDocumentCompletedEventArgs e)
		{
			if (_lbInfo.Document != null)
			{
				_lbInfo.Document.Click += Document_Click;
				_lbInfo.Document.Body.Style = "font-family: Verdana, Arial, Helvetica, sans-serif; " +
					"font-size: 7.5pt; margin: 2pt; padding: 0pt; background-color: #ffffee; cursor: default;";
			}
		}

		/// <summary>
		/// Handles the Web browser document's Click event.
		/// </summary>
		private void Document_Click(object sender, HtmlElementEventArgs e)
		{
			e.ReturnValue = true;

			HtmlElement pointed = _lbInfo.Document.GetElementFromPoint(e.ClientMousePosition);
			if (pointed == null)
				return;

			if (pointed.TagName.ToLower() == "a")
			{
				string href = pointed.GetAttribute("href").ToLower();
				if (href.StartsWith("help:"))
				{
					string topic = href.Substring(5);
					string helpFile = ResolveHelpPath(_appPath);

					if (System.IO.File.Exists(helpFile))
					{
						Help.ShowHelp(this, helpFile, HelpNavigator.Topic, topic);

						// Prevent link follow
						e.ReturnValue = false;
					}
				}
			}
		}

		private static string ResolveHelpPath(string baseDir)
		{
			string[] helpFiles = new string[]
			{
				"MindFusion.Scheduling.chm",
				"MindFusion WinForms.chm"
			};

			System.IO.DirectoryInfo dirInfo = new System.IO.DirectoryInfo(baseDir);
			int i = 0;
			while (dirInfo != null)
			{
				string helpFile = System.IO.Path.Combine(dirInfo.FullName, helpFiles[i]);
				if (System.IO.File.Exists(helpFile))
					return helpFile;

				i++;
				if (i == helpFiles.Length)
				{
					i = 0;
					dirInfo = dirInfo.Parent;
				}
			}

			return string.Empty;
		}

		private void OnAbout(object sender, System.EventArgs e)
		{
			System.Reflection.Assembly assembly = System.Reflection.Assembly.GetAssembly(
				typeof(MindFusion.Scheduling.WinForms.Calendar));

			if (assembly == null)
				return;

			Type aboutFormType = assembly.GetType(
				"MindFusion.Scheduling.WinForms.AboutForm");

			if (aboutFormType == null)
				return;

			((Form)Activator.CreateInstance(aboutFormType)).ShowDialog(this);
		}

		private void OnExit(object sender, System.EventArgs e)
		{
			this.Close();
		}

		private void calendar_Draw(object sender, DrawEventArgs e)
		{
			if (e.Element == CustomDrawElements.CellHeader)
			{
				Rectangle bounds = e.Bounds;
				if (e.Date.Date == DateTime.Today)
				{
					using (Brush brush = new SolidBrush(Color.FromArgb(100, Color.White)))
					{
						e.Graphics.FillRectangle(brush, bounds);
					}

					using (Pen pen = new Pen(Color.Red, 3))
					{
						pen.Alignment = PenAlignment.Inset;
						e.Graphics.DrawRectangle(pen, bounds);
					}
				}
				else
				{
					if (holidays != null && holidays.Length > 0)
					{
						bool isHoliday = false;
						foreach (Holiday holiday in holidays)
						{
							if (holiday.Date <= e.Date.Date && e.Date.Date <= holiday.EndDate)
							{
								isHoliday = true;
								break;
							}
						}

						if (isHoliday)
						{
							using (Brush brush = new SolidBrush(Color.FromArgb(128, Color.LightSteelBlue)))
							{
								e.Graphics.FillRectangle(brush, bounds);
							}

							using (Pen pen = new Pen(Color.FromArgb(192, Color.SlateGray)))
							{
								Rectangle borderBounds = bounds;
								borderBounds.Width -= 1;
								borderBounds.Height -= 1;
								e.Graphics.DrawRectangle(pen, borderBounds);
							}

							using (StringFormat format = new StringFormat())
							{
								format.Alignment = StringAlignment.Center;
								format.LineAlignment = StringAlignment.Center;

								using (Brush brush = new SolidBrush(Color.FromArgb(192, 0, 0)))
								{
									e.Graphics.DrawString(e.Text, e.Style.HeaderFont, brush,
										new RectangleF(bounds.X, bounds.Y, bounds.Width, bounds.Height), format);
								}
							}
						}
					}
				}
			}
		}

		private void calendar_TooltipDisplaying(object sender, TooltipEventArgs e)
		{
			StringBuilder tooltip = new StringBuilder();
			if (holidays != null && holidays.Length > 0)
			{
				foreach (Holiday holiday in holidays)
				{
					if (holiday.Date <= e.Time.Date && e.Time.Date <= holiday.EndDate)
					{
						if (tooltip.Length > 0)
							tooltip.AppendLine();
						tooltip.Append(holiday.Title);
					}
				}
			}

			e.Tooltip = tooltip.ToString();
		}

		private void calendar_VisibleDateChanged(object sender, DateChangedEventArgs e)
		{
			UpdateHolidays();
		}

		private void UpdateHolidays()
		{
			DateTime date = calendar.Date;
			holidays = new USHolidayProvider().GetHolidays(
				new DateTime(date.Year, date.Month, 1),
				new DateTime(date.Year, date.Month, DateTime.DaysInMonth(date.Year, date.Month)));
		}


		private string _appPath;
		private Holiday[] holidays;
	}
}
